from flask import current_app
import tweepy

class Tweepy(object):
    """The :class:`~flask_tweepy.Tweepy` class configures and creates a
    :class:`tweepy.API` instance based on Flask configuration variables.
    Instantiating a :class:`~flask_tweepy.Tweepy` with no arguments does not
    configure the :class:`tweepy.API`, and will not be usable unless
    :meth:`Tweepy.init_app` is subsequently called.

    :param flask.Flask app: the Flask application to which this Tweepy
       should be attached
    :param str config_prefix: the configuration variable prefix
    """

    def __init__(self, app=None, config_prefix='TWEEPY'):
        # this is overridden by init_app if app is supplied
        self.config_prefix = None
        if app is not None:
            self.init_app(app, config_prefix)

    def init_app(self, app, config_prefix='TWEEPY'):
        """Initialize this :class:`~flask_tweepy.Tweepy` if no arguments were
        given to the constructor. Accepts the same arguments as the
        constructor.
        """
        app.extensions.setdefault('tweepy', {})

        if config_prefix in app.extensions['tweepy']:
            raise Exception('duplicate config_prefix "%s"' % config_prefix)

        self.config_prefix = config_prefix

        consumer_key = app.config.get('%s_CONSUMER_KEY' % config_prefix)
        consumer_secret = app.config.get('%s_CONSUMER_SECRET' % config_prefix)
        token_key = app.config.get('%s_ACCESS_TOKEN_KEY' % config_prefix)
        token_secret = app.config.get('%s_ACCESS_TOKEN_SECRET' % config_prefix)

        if not all((consumer_key, consumer_secret)):
            raise Exception('%s_CONSUMER_KEY and %s_CONSUMER_SECRET '
                            'are required' % (config_prefix, config_prefix))

        # TODO 0.2: allow tweets as users of the app,
        # so don't require token_key and token_secret
        if not all((token_key, token_secret)):
            raise Exception('%s_ACCESS_TOKEN_KEY and %s_ACCESS_TOKEN_SECRET '
                            'are required' % (config_prefix, config_prefix))

        # TODO 0.2: support callback_url
        auth = tweepy.auth.OAuthHandler(consumer_key, consumer_secret)
        if all((token_key, token_secret)):
            auth.set_access_token(token_key, token_secret)

        api = tweepy.API(auth)
        app.extensions['tweepy'][config_prefix] = (auth, api)

    @property
    def api(self):
        """Return the :class:`tweepy.API` object if Flask-Tweepy has been
        properly configured and initialized, else return None.
        """
        tweepies = current_app.extensions.get('tweepy', {})
        auth, api = tweepies.get(self.config_prefix, (None, None))
        return api




import unittest

import flask
from flask.ext.tweepy import Tweepy

class FlaskTweepyConfigTest(unittest.TestCase):

    def setUp(self):
        self.app = flask.Flask('test')
        self.context = self.app.test_request_context('/')
        self.context.push()

    def tearDown(self):
        self.context.pop()

    def test_default_config_prefix(self):
        self.app.config['TWEEPY_CONSUMER_KEY'] = 'keykeykey'
        self.app.config['TWEEPY_CONSUMER_SECRET'] = 'secretsecret'
        self.app.config['TWEEPY_ACCESS_TOKEN_KEY'] = 'keykeykey'
        self.app.config['TWEEPY_ACCESS_TOKEN_SECRET'] = 'secretsecret'

        tweepy = Tweepy(self.app)
        assert tweepy.api is not None

    def test_custom_config_prefix(self):
        self.app.config['CUSTOM_CONSUMER_KEY'] = 'keykeykey'
        self.app.config['CUSTOM_CONSUMER_SECRET'] = 'secretsecret'
        self.app.config['CUSTOM_ACCESS_TOKEN_KEY'] = 'keykeykey'
        self.app.config['CUSTOM_ACCESS_TOKEN_SECRET'] = 'secretsecret'

        tweepy = Tweepy(self.app, config_prefix='CUSTOM')
        assert tweepy.api is not None

    def test_multiple_tweepys(self):
        self.app.config['TWEEPY_CONSUMER_KEY'] = 'keykeykey'
        self.app.config['TWEEPY_CONSUMER_SECRET'] = 'secretsecret'
        self.app.config['TWEEPY_ACCESS_TOKEN_KEY'] = 'keykeykey'
        self.app.config['TWEEPY_ACCESS_TOKEN_SECRET'] = 'secretsecret'

        tweepy1 = Tweepy(self.app)
        assert tweepy1.api is not None

        self.app.config['CUSTOM_CONSUMER_KEY'] = 'keykeykey'
        self.app.config['CUSTOM_CONSUMER_SECRET'] = 'secretsecret'
        self.app.config['CUSTOM_ACCESS_TOKEN_KEY'] = 'keykeykey'
        self.app.config['CUSTOM_ACCESS_TOKEN_SECRET'] = 'secretsecret'

        tweepy2 = Tweepy(self.app, config_prefix='CUSTOM')
        assert tweepy2.api is not None

        assert tweepy1.api is not tweepy2.api

    def test_fails_without_required_configs(self):
        self.app.config['TWEEPY_CONSUMER_KEY'] = 'keykeykey'
        # self.app.config['TWEEPY_CONSUMER_SECRET'] = 'secretsecret'
        self.app.config['TWEEPY_ACCESS_TOKEN_KEY'] = 'keykeykey'
        self.app.config['TWEEPY_ACCESS_TOKEN_SECRET'] = 'secretsecret'

        self.assertRaises(Exception, Tweepy, self.app)

    def test_api_is_none_if_unconfigured(self):
        tweepy = Tweepy()

        assert tweepy.api is None

    def test_init_app(self):
        tweepy = Tweepy()

        self.app.config['TWEEPY_CONSUMER_KEY'] = 'keykeykey'
        self.app.config['TWEEPY_CONSUMER_SECRET'] = 'secretsecret'
        self.app.config['TWEEPY_ACCESS_TOKEN_KEY'] = 'keykeykey'
        self.app.config['TWEEPY_ACCESS_TOKEN_SECRET'] = 'secretsecret'

        tweepy.init_app(self.app)
        assert tweepy.api is not None

    def test_fails_with_duplicate_config_prefix(self):
        self.app.config['TWEEPY_CONSUMER_KEY'] = 'keykeykey'
        self.app.config['TWEEPY_CONSUMER_SECRET'] = 'secretsecret'
        self.app.config['TWEEPY_ACCESS_TOKEN_KEY'] = 'keykeykey'
        self.app.config['TWEEPY_ACCESS_TOKEN_SECRET'] = 'secretsecret'

        Tweepy(self.app)
        self.assertRaises(Exception, Tweepy, self.app)




"""
Flask-Tweepy
------------

Tweet easily from Flask applications.
"""

from setuptools import setup

setup(
    name='Flask-Tweepy',
    version='0.1',
    url='http://flask-tweepy.readthedocs.org/',
    license='BSD',
    author='Dan Crosta',
    author_email='dcrosta@late.am',
    description='Tweet easily from Flask applications',
    long_description=__doc__,
    zip_safe=False,
    platforms='any',
    py_modules=['flask_tweepy'],
    install_requires=[
        'Flask >= 0.8',
        'tweepy >= 1.8, <1.9',
    ],
    classifiers=[
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
        'Topic :: Software Development :: Libraries :: Python Modules'
    ],
    tests_require=[
        'nose',
    ],
    test_suite='nose.collector',
)

